# User's Guide

This section provides orientation on using zavin-as.  


## Motivation
- Ability to create independent, deployable web app within a server
- Define global resource and let allow overriding from the application
- Use tried and tested tools within the Node.js/Express ecosystem


## System Configuration
zavin-as includes a system configuration defined in the _config.json_ file which you can change such as enabling certain functions or location of your database. It is recommended that you setup your user-defined system configuration tailored to your needs.  By default zavin-as uses the default factory settings defined in the zavin-as/etc/config.json file. To customize it simply copy the file inside the .zavin/ defined in your home directory and make your changes from there. When your done restart the server for the changes to take effect. The config.json uses JSON format thus you must follow the convention of setting values. Checkout out http://json.org for more details

### System Configuration Options

| Property              | Description
| --------------------- | ---------------------------
| `datasource_dev`      | Database URL for development. Default is `zavin_dev_db` localhost with port 27017
| `datasource_prod`     | Database URL for development. Default is `zavin_db` localhost with port 27017
| `enable_browser_sync` | Set to `true` to enable browser sync feature. Note however, this only works when in development mode.
| `enable_replica_set`  | Set to `true` to enable replica set support. Default: `false`
| `enable_ssl`          | Indicate if SSL/TLS is enabled. Default: `true`
| `port_dev`            | Port number for used during development. Default: `3000`
| `port_prod`           | Port number for used during production. Default: `8080`
| `port_ssl`            | SSL Port number redirect from `port_dev` or `port_prod`. Default: `8443`
| `signed_cert_path`    | Location of signed certificate file. Default uses the pre-defined path in `./etc/ckdynamics-cert.pem` used in development.  You need to replace this when deploying to a public environment.
| `signed_key_path`     | Location of signed certificate key file. Default uses the pre-defined path in `./etc/ckdynamics-key.pem` used in development.  You need to replace this when deploying to a public environment.


## Performance and Security
zavin-as uses Express as its web framework. This setup includes additional modules which adds performance and security functions. Refer to the _package.json_ file to see the list of dependent modules. Notable performance and security functions includes:

- [compression](https://github.com/expressjs/compression#readme)
- [bcrypt-nodejs](https://github.com/shaneGirish/bcryptJS#readme)
- [express-force-ssl](http://github.com/battlejj/express-force-ssl)
- [express-rate-limit](https://github.com/nfriedly/express-rate-limit)
- [helmet](https://github.com/helmetjs/helmet#readme)
- [passport](http://passportjs.org/)
- [passport-local](https://github.com/jaredhanson/passport-local#readme)

Other modules which can be used in your app:

- [csvjson](https://github.com/pradeep-mishra/csvjson)
- [ejs](https://github.com/mde/ejs)
- [fs-extra](https://github.com/jprichardson/node-fs-extra)
- [lodash](https://lodash.com/)
- [minimist](https://github.com/substack/minimist)
- [moment](http://momentjs.com)
- [winston](https://github.com/winstonjs/winston#readme)


### Certificate
A default self-signed certificate is included for use during development. Testing HTTPS during development can be done by creating certificate in your local machine.  If you don’t already have certificate you need to generate one

```
openssl req -x509 -newkey rsa:2048 -keyout ckdynamics-key.pem -out ckdynamics-cert.pem -days 365
```

You will be asked a couple of questions…

```
         Country: PH
  State/Province: Laguna
        Locality: Binan
    Organization: CNK Business Dynamics Co.
       Unit Name: ckdynamics
     Common Name: ckdynamics
   Email Address: admin@ckdynamics.com.ph
```

Passphrase is `ckdynamics.com.ph`.

In case of error such as:

```
Error: error:0906A068:PEM routines:PEM_do_header:bad password read...
```

you need to remove the pass:

```
openssl rsa -in ckdynamics-key.pem -out newkey.pem && mv newkey.pem ckdynamics-key.pem
```

__Ignoring "Error: self-signed certificate"__

As self-signed certificte is generated this is typically used during development and tesing only. You may encounter error such as "Error: self-signed certificate" as the system is setup to require authorized certificate.  To temporarily ignore the warning/error message add the following before making https.requests:

```
process.env.NODE_TLS_REJECT_UNAUTHORIZED = "0";
```

Sample use case of this is found in `test/` directory.


### Secure Connection (SSL/TLS)


### Using BrowserSync

__Quirks__

Using browser-sync is done only during development and with SSL enabled. This is to ensure optimal development and testing targeted in a production environment where HTTPS is necessary.  When making changes in js files Google Chrome will open a new tab which sometimes can be annoying. As a workaround, you could use Safari browser which doesn't open new tabs during file changes. Note that HTML and CSS has no effect on this.


### MongoDB Database
